#include <nds.h>
#include <nds/debug.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "stacktrace.h"


void PrintStacktrace()
{
  unsigned int n;
  unsigned int count;
  StacktraceEntry entries[32]; // allocate memory for 32 stack trace entries

  // get only the 10 deepest stack trace entries
  count = Stacktrace(entries, 10);

  printf(" ADDRESS    FUNCTION\n\n");

  for (n=0; n<count; ++n)
    printf(" 0x%08x %s\n", entries[n].addr, entries[n].name);

  printf(" ------------------------------\n");
}

__attribute__((noinline))	
static void ThirdFunc()
{
  char text[99];
  sprintf(text, "%s\n", "Even more!");
  printf(text);

  consoleClear();
  PrintStacktrace();

  // when you compile with -O4 this function will
  // directly branch back to main.
}


static void SecondFunc()
{
  char text[77];
  sprintf(text, "%s\n", "More stack usage");

  int i=0;
  while(TRUE) // infinite loop
  {
    printf(text);
    if(++i==7)
      break; // early out
  }

  ThirdFunc();
  sprintf(text, "%s\n", "A bit stack usage");
  printf(text);
}

void FirstFunc()
{
  char text[0x1234]; // generates "add sp, pc+ofs" instruction
  char text2[64]; // generates "add sp, pc+ofs" instruction
  sprintf(text, "%s\n", "A bit stack usage");
  printf(text); // use text so the optimizer cannot remove it

  SecondFunc();

  sprintf(text2, "%s\n", "A bit stack usage bye bye");
  printf(text2);
}

static void OutputDebugString(const char* format, ...)
{
  va_list args;
  char    buffer[128]; // increase to support more characters

  va_start(args,0);
  vsnprintf(buffer, sizeof(buffer), format, args);
  va_end(args);

  // replace this call with the print routine of your choice
  nocashMessage(buffer);
}


int main(void)
{
  consoleDemoInit();

  OutputDebugString("main = 0x%08x\n", &main);
  OutputDebugString("FirstFunc = 0x%08x\n", &FirstFunc);
  OutputDebugString("SecondFunc = 0x%08x\n", &SecondFunc);
  OutputDebugString("ThirdFunc = 0x%08x\n", &ThirdFunc);

  FirstFunc();
  
	while(1)
  {
		swiWaitForVBlank();
	}

	return 0;
}
